package com.runner.tradeFinace;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import config.ClientConfig;
import config.ProjectResource;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openpgp.PGPPrivateKey;
import org.bouncycastle.openpgp.PGPSecretKey;
import org.bouncycastle.openpgp.PGPSecretKeyRingCollection;
import org.bouncycastle.openpgp.PGPUtil;
import org.bouncycastle.openpgp.operator.jcajce.JcaKeyFingerprintCalculator;
import util.JsonUtil;
import util.PGPEncryptionExample;
import util.PGPExampleUtil;
import util.SampleApiRequest;

import java.io.InputStream;
import java.security.Security;

public class ClientSideDecryption {

    /**
     * used to fix java.security.NoSuchProviderException: no such provider: BC
     */
    static {
        try {
            Security.addProvider(new BouncyCastleProvider());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public ClientSideDecryption() {


    }

    public static void main(String[] args) {
        ClientSideDecryption clientSideDecryption = new ClientSideDecryption();
        clientSideDecryption.decryptResponse();
    }

    public static InputStream findFile(final String file) {
        return SampleApiRequest.class.getClassLoader().getResourceAsStream(file);
    }

    public static String jsonFormat(String jsonString) {
        JSONObject object = JSONObject.parseObject(jsonString);
        jsonString = JSON.toJSONString(object, SerializerFeature.PrettyFormat, SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteDateUseDateFormat);
        return jsonString;
    }

    public void decryptResponse() {

        try {
            String targetHost = ClientConfig.targetHost;
            System.out.println("----- #Info# -----");
            System.out.println("Target Host:" + targetHost);
            System.out.println("----- #Start of Client Side Token And Encryption Steps# -----");
            String clientPrivateKeyFile = "";
            String clientPrivateKeyPWD = "";
            PGPPrivateKey clientPgpPrivateKey;
            PGPSecretKeyRingCollection pgpSec;
            PGPSecretKey clientPgpSecretKey;
            switch (targetHost) {
                case "SANDBOX":
                    clientPrivateKeyFile = ProjectResource.Sandbox.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                    clientPrivateKeyPWD = ClientConfig.Sandbox.CLIENT_PRV_KEY_PWD;
                    break;
                case "UAT":
                    clientPrivateKeyFile = ProjectResource.Uat.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                    clientPrivateKeyPWD = ClientConfig.Uat.CLIENT_PRV_KEY_PWD;
                    break;
                case "PRODUCTION":
                    clientPrivateKeyFile = ProjectResource.Production.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                    clientPrivateKeyPWD = ClientConfig.Production.CLIENT_PRV_KEY_PWD;
                    break;
                default:
                    throw new IllegalStateException("Unexpected value: " + targetHost);
            }
            // Load Client Private PGP Keys
            pgpSec = new PGPSecretKeyRingCollection(
                    PGPUtil.getDecoderStream(findFile(clientPrivateKeyFile)), new JcaKeyFingerprintCalculator());

            clientPgpSecretKey = PGPExampleUtil
                    .readSecretKey(ProjectResource.RESOURCE_PATH.concat(clientPrivateKeyFile));
            clientPgpPrivateKey = PGPExampleUtil.findSecretKey(pgpSec, clientPgpSecretKey.getKeyID(),
                    clientPrivateKeyPWD.toCharArray());

            // response payload
            String bankResponseEncrpytedBase64String = JsonUtil.getEncryptedBankResponse();
            System.out.println("#5.1 Response Message (Base64 Encoded):\n" + bankResponseEncrpytedBase64String);
            ;
            System.out.println("#6. Decrypting Response Message from #5.1...");
            String decrypt = PGPEncryptionExample.decryptResponseMessage(clientPgpPrivateKey, bankResponseEncrpytedBase64String);
            String jsonFormat = jsonFormat(decrypt);
            System.out.println("#6.1 Decrypted Response Message:");
            System.out.println(jsonFormat);
            System.out.println("----- #End of Client Side Decryption Steps# -----");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

}
