package com.runner.tradeFinace;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import config.ClientConfig;
import config.ProjectResource;
import config.product.tradeFinance.Common;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openpgp.*;
import org.bouncycastle.openpgp.operator.jcajce.JcaKeyFingerprintCalculator;
import org.bouncycastle.openpgp.operator.jcajce.JcaPGPKeyConverter;
import util.*;

import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Security;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ClientSideTokenNEncryption {

    /**
     * used to fix java.security.NoSuchProviderException: no such provider: BC
     */
    static {
        try {
            Security.addProvider(new BouncyCastleProvider());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public ClientSideTokenNEncryption() {

    }

    public static void main(String[] args) {
        ClientSideTokenNEncryption clientTokenNEncrypt = new ClientSideTokenNEncryption();
        JSONObject requestHeaderJson = new JSONObject();
        String unencryptedRequest = "";
        try {
            encryptPayloadAndMakeToken(requestHeaderJson, unencryptedRequest);
        } catch (NoSuchProviderException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (PGPException e) {
            e.printStackTrace();
        }
    }

    public static Map<String, String> encryptPayloadAndMakeToken(JSONObject requestHeaderJson, String unencryptedRequest)
            throws IOException, PGPException, NoSuchProviderException, NoSuchAlgorithmException {
        String targetHost = ClientConfig.targetHost;
        String apiFolder = Common.API_FOLDER;
        System.out.println("----- #Info# -----");
        // 1. Prepare Request to HSBC API to HSBC API Services
        System.out.println("Target Host:" + targetHost);
        unencryptedRequest = apiFolder.concat(Common.API_INFO.get("COM")[2]);
        System.out.println("----- #Start of Client Side Token And Encryption Steps# -----");
        // 1. Prepare Request to HSBC API to HSBC API Services
        JSONObject requestJson = JsonUtil.getRequestJson(unencryptedRequest.concat(".json"));

        // 1.1. Reads Plain Request JSON

        // 1.2. Converts JSON to STRING
        String requestPayloadPlainString = JSON.toJSONString(requestJson);
        System.out.println("#1.2 Converting following Unencrypted Content as JSONString...\n" + JSON.toJSONString(requestJson, SerializerFeature.PrettyFormat, SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteDateUseDateFormat));
        System.out.println();

        // 2. Encrypts Request Payload
        System.out.println("#2. Start of preparing the HTTP Request Payload... ");
        // 2.1. Load HSBC PGP Public Key
        System.out.println("#2.1 Loading Security Credentials... ");
        String bankPublicKeyFile = "";
        String clientPrivateKeyFile = "";
        String clientPrivateKeyPWD = "";
        PGPPublicKey hsbcPgpPublicKey;
        PGPPrivateKey clientPgpPrivateKey;
        PGPSecretKeyRingCollection pgpSec;
        PGPSecretKey clientPgpSecretKey;
        switch (targetHost) {
            case "SANDBOX":
                bankPublicKeyFile = ProjectResource.Sandbox.CLIENT_KEY_PATH.concat(ProjectResource.HSBC_PUB_KEY).concat(".asc");
                clientPrivateKeyFile = ProjectResource.Sandbox.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                clientPrivateKeyPWD = ClientConfig.Sandbox.CLIENT_PRV_KEY_PWD;
                break;
            case "UAT":
                bankPublicKeyFile = ProjectResource.Uat.CLIENT_KEY_PATH.concat(ProjectResource.HSBC_PUB_KEY).concat(".asc");
                clientPrivateKeyFile = ProjectResource.Uat.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                clientPrivateKeyPWD = ClientConfig.Uat.CLIENT_PRV_KEY_PWD;
                break;
            case "PRODUCTION":
                bankPublicKeyFile = ProjectResource.Production.CLIENT_KEY_PATH.concat(ProjectResource.HSBC_PUB_KEY).concat(".asc");
                clientPrivateKeyFile = ProjectResource.Production.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                clientPrivateKeyPWD = ClientConfig.Production.CLIENT_PRV_KEY_PWD;
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + targetHost);
        }
        System.out.println("{");
        System.out.println("  \"bankPublicKeyFile\": \"".concat(bankPublicKeyFile).concat("\","));
        System.out.println("  \"clientPrivateKeyFile\": ".concat(clientPrivateKeyFile).concat("\""));
        System.out.println("}");
        System.out.println("\n#2.2 Reading PGP Keys...");
        hsbcPgpPublicKey = PGPExampleUtil
                .readPublicKey(ProjectResource.RESOURCE_PATH.concat(bankPublicKeyFile));
        pgpSec = new PGPSecretKeyRingCollection(
                PGPUtil.getDecoderStream(findFile(clientPrivateKeyFile)), new JcaKeyFingerprintCalculator());
        clientPgpSecretKey = PGPExampleUtil
                .readSecretKey(ProjectResource.RESOURCE_PATH.concat(clientPrivateKeyFile));
        clientPgpPrivateKey = PGPExampleUtil.findSecretKey(pgpSec, clientPgpSecretKey.getKeyID(),
                clientPrivateKeyPWD.toCharArray());
        List<PGPPublicKey> pgpPublicKeys = new ArrayList<PGPPublicKey>();
        pgpPublicKeys.add(hsbcPgpPublicKey);
        String clientKeyId = (Long.toHexString(clientPgpPrivateKey.getKeyID()).toUpperCase());

        // 2.2. Encrypts Request Payload with HSBC Public Key
        Map<String, String> encryptedPayloadAndToken = new HashMap<String, String>();
        encryptedPayloadAndToken.put("requestPayloadEncryptedBase64String", PGPEncryptionExample
                .encrpytRequestPayload(requestPayloadPlainString, pgpPublicKeys, true, true, false));

        // 3. Prepares Digital Signature (JSON Web Token)
        String APIProfileId;
        switch (targetHost) {
            case "SANDBOX":
                APIProfileId = ClientConfig.Sandbox.API_PROFILE_ID;
                break;
            case "UAT":
                APIProfileId = ClientConfig.Uat.API_PROFILE_ID;
                break;
            case "PRODUCTION":
                APIProfileId = ClientConfig.Production.API_PROFILE_ID;
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + targetHost);
        }
        System.out.println("\n#3. Start of preparing the Digital Signature (JSON Web Token)");
        // 3.1. Loads Client Private PGP Keys

        // 3.2. Converts Client PGP Private key to RSA key
        PrivateKey clientPrivateKey = new JcaPGPKeyConverter().getPrivateKey(clientPgpPrivateKey);

        // 3.3. Generates token - JWT
        encryptedPayloadAndToken.put("token", BearerToken.createToken(ClientConfig.TOKEN_TYP, clientKeyId, ClientConfig.TOKEN_ALG, APIProfileId, ClientConfig.TOKEN_AUD, ClientConfig.TOKEN_VER, encryptedPayloadAndToken.get("requestPayloadEncryptedBase64String"), ClientConfig.TOKEN_PAYLOAD_HASH_ALG, clientPrivateKey));
        System.out.println("\n#4. JWS Token and Encrypted Payload:");
        System.out.println("#4.1 JSON Web Token to be put in the Authorization Header:\n".concat("JWS ").concat(encryptedPayloadAndToken.get("token").concat("\n")));
        System.out.println("#4.2 Encrypted Request Payload:\n".concat(encryptedPayloadAndToken.get("requestPayloadEncryptedBase64String").concat("\n")));
        return encryptedPayloadAndToken;
    }

    public static InputStream findFile(final String file) {
        return SampleApiRequest.class.getClassLoader().getResourceAsStream(file);
    }

}
