package util;

import com.alibaba.fastjson.JSONObject;
import config.ClientConfig;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpStatus;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;

import javax.net.ssl.SSLContext;
import java.io.IOException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;
import java.util.UUID;

public class HttpClientUtil {

    /**
     * get request.
     *
     * @return
     */
    public static String doPost(String url, JSONObject requestHeaderJson, String encryptedRequestBase64, String token)
            throws Exception {
        System.out.println("\n#4 Preparing API Request... ");
        HttpPost post = new HttpPost(url);
        HttpHost proxyHost;
        RequestConfig config;
        //RequestConfig config = RequestConfig.custom().build();
        JSONObject jsonheader = getHeaders(requestHeaderJson, token);
        System.out.println("#4.1 Request Header:");
        for (Map.Entry<String, Object> entry : jsonheader.entrySet()) {
            post.setHeader(entry.getKey(), (String) entry.getValue());
            System.out.println(entry.getKey() + ":" + entry.getValue());
        }
        System.out.println("\n#4.2 Request Body (bankPublicKeyEncryptedBase64):\n".concat(encryptedRequestBase64));
        post.setEntity(new StringEntity(encryptedRequestBase64, "UTF-8"));
        SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
            public boolean isTrusted(X509Certificate[] arg0, String arg1) throws CertificateException {
                return true;
            }
        }).build();
        SSLConnectionSocketFactory sslcsf = new SSLConnectionSocketFactory(sslContext,
                SSLConnectionSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
        String resJson = null;
        System.out.println("\n#4.3 Sending Request to HSBC API Endpoint ".concat(url));
        switch (ClientConfig.proxyMode) {
            case (0):
                config = RequestConfig.custom().build();
                post.setConfig(config);
                try (CloseableHttpClient client = HttpClients.custom().setSSLSocketFactory(sslcsf).build()) {
                    CloseableHttpResponse response = client.execute(post);
                    HttpEntity entity = response.getEntity();
                    resJson = EntityUtils.toString(entity);
                    break;
                } catch (Exception e) {
                    e.printStackTrace();
                }
            case (1):
                proxyHost = new HttpHost(ClientConfig.proxyHost, ClientConfig.proxyPort);
                config = RequestConfig.custom().setProxy(proxyHost).build();
                post.setConfig(config);
                try (CloseableHttpClient client = HttpClients.custom().setSSLSocketFactory(sslcsf).build()) {
                    CloseableHttpResponse response = client.execute(post);
                    HttpEntity entity = response.getEntity();
                    resJson = EntityUtils.toString(entity);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                break;
            case (2):
                proxyHost = new HttpHost(ClientConfig.proxyHost, ClientConfig.proxyPort);
                CredentialsProvider credsProvider = new BasicCredentialsProvider();
                //Setting the credentials
                credsProvider.setCredentials(new AuthScope(ClientConfig.proxyHost, ClientConfig.proxyPort),
                        new UsernamePasswordCredentials(ClientConfig.proxyUser, ClientConfig.proxyPassword));
                config = RequestConfig.custom().setProxy(proxyHost).build();
                post.setConfig(config);
                try (CloseableHttpClient client = HttpClients.custom().setSSLSocketFactory(sslcsf).setDefaultCredentialsProvider(credsProvider).build()) {
                    CloseableHttpResponse response = client.execute(post);
                    HttpEntity entity = response.getEntity();
                    resJson = EntityUtils.toString(entity);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + ClientConfig.proxyMode);
        }
        return resJson;
    }

    public static String doGet(String url, JSONObject requestHeaderJson, String token, String profileId) {
        String strResult = null;
        try {
            HttpGet requestGet = new HttpGet(url);
            JSONObject jsonheader = getHeaders(requestHeaderJson, token);
            for (Map.Entry<String, Object> entry : jsonheader.entrySet()) {
                requestGet.setHeader(entry.getKey(), (String) entry.getValue());
            }
            CloseableHttpClient client = HttpClients.createDefault();
            CloseableHttpResponse response = client.execute(requestGet);
            HttpEntity httpEntity = response.getEntity();
            if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                strResult = EntityUtils.toString(response.getEntity());
                System.out.println("response : " + strResult);
                return strResult;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return strResult;
    }

    public static String doGet(String url, JSONObject requestHeaderJson, String token) {
        return doGet(url, requestHeaderJson, token, null);
    }

    public static JSONObject getHeaders(JSONObject requestHeaderJson, String token) {
        JSONObject jsonheader = new JSONObject();
        requestHeaderJson.put("Authorization", "JWS ".concat(token));
        requestHeaderJson.put("requestId", getUuid());
        requestHeaderJson.put("requestTime", getCurrentUtcTime());
        return requestHeaderJson;
    }

    private static String getUuid() {
        return String.valueOf(UUID.randomUUID()).replace("-", "");
    }

    private static String getCurrentUtcTime() {
        ZonedDateTime utcDateTime = ZonedDateTime.now(ZoneOffset.UTC);
        return utcDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }
}
