package util;
/*
 * ***************************************************************
 * , Copyright. HSBC Holdings plc ccyy ALL RIGHTS RESERVED.
 *
 * This software is only to be used for the purpose for which it
 * has been provided. No part of it is to be reproduced,
 * disassembled, transmitted, stored in a retrieval system or
 * translated in any human or computer language in any way or
 * for any other purposes whatsoever without the prior written
 * consent of HSBC Holdings plc.
 * ***************************************************************
 */

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import config.ClientConfig;
import config.ProjectResource;
import org.bouncycastle.openpgp.*;
import org.bouncycastle.openpgp.operator.jcajce.JcaKeyFingerprintCalculator;
import org.bouncycastle.openpgp.operator.jcajce.JcaPGPKeyConverter;

import java.io.InputStream;
import java.security.PrivateKey;
import java.util.ArrayList;
import java.util.List;


public class SampleApiRequest {

    public static void ExecuteRequest(String apiName, String endpointUrl, JSONObject requestHeaderJson, String unencryptedRequest) throws Exception {

        String targetHost = ClientConfig.targetHost;
        System.out.println("----- #Info# -----");
        // 1. Prepare Request to HSBC API to HSBC API Services
        System.out.println("Target Host:" + targetHost);
        System.out.println("API Name: " + apiName);
        System.out.println("----- #Start of API Sample Code Steps# -----");
        System.out.println("#1. Preparing Unencrypted Request Content");
        // 1.1. Reads Plain Request JSON
        JSONObject requestJson = JsonUtil.getRequestJson(unencryptedRequest.concat(".json"));
        System.out.println("#1.1 Reading the Request Content from: " + ProjectResource.RESOURCE_PATH + unencryptedRequest.concat(".json"));
        // 1.2. Converts JSON to STRING
        String requestPayloadPlainString = JSON.toJSONString(requestJson);
        System.out.println("#1.2 Converting following Unencrypted Content as JSONString...\n" + JSON.toJSONString(requestJson, SerializerFeature.PrettyFormat, SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteDateUseDateFormat));
        System.out.println();

        // 2. Encrypts Request Payload
        System.out.println("#2. Start of preparing the HTTP Request Payload... ");
        // 2.1. Load HSBC PGP Public Key
        System.out.println("#2.1 Loading Security Credentials... ");
        String bankPublicKeyFile = "";
        String clientPrivateKeyFile = "";
        String clientPrivateKeyPWD = "";
        PGPPublicKey hsbcPgpPublicKey;
        PGPPrivateKey clientPgpPrivateKey;
        PGPSecretKeyRingCollection pgpSec;
        PGPSecretKey clientPgpSecretKey;
        switch (targetHost) {
            case "SANDBOX":
                bankPublicKeyFile = ProjectResource.Sandbox.CLIENT_KEY_PATH.concat(ProjectResource.HSBC_PUB_KEY).concat(".asc");
                clientPrivateKeyFile = ProjectResource.Sandbox.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                clientPrivateKeyPWD = ClientConfig.Sandbox.CLIENT_PRV_KEY_PWD;
                break;
            case "UAT":
                bankPublicKeyFile = ProjectResource.Uat.CLIENT_KEY_PATH.concat(ProjectResource.HSBC_PUB_KEY).concat(".asc");
                clientPrivateKeyFile = ProjectResource.Uat.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                clientPrivateKeyPWD = ClientConfig.Uat.CLIENT_PRV_KEY_PWD;
                break;
            case "PRODUCTION":
                bankPublicKeyFile = ProjectResource.Production.CLIENT_KEY_PATH.concat(ProjectResource.HSBC_PUB_KEY).concat(".asc");
                clientPrivateKeyFile = ProjectResource.Production.CLIENT_KEY_PATH.concat(ProjectResource.CLIENT_PRV_KEY).concat(".asc");
                clientPrivateKeyPWD = ClientConfig.Production.CLIENT_PRV_KEY_PWD;
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + targetHost);
        }
        System.out.println("{");
        System.out.println("  \"bankPublicKeyFile\": \"".concat(bankPublicKeyFile).concat("\","));
        System.out.println("  \"clientPrivateKeyFile\": ".concat(clientPrivateKeyFile).concat("\""));
        System.out.println("}");
        System.out.println("\n#2.2 Reading PGP Keys...");
        hsbcPgpPublicKey = PGPExampleUtil
                .readPublicKey(ProjectResource.RESOURCE_PATH.concat(bankPublicKeyFile));
        pgpSec = new PGPSecretKeyRingCollection(
                PGPUtil.getDecoderStream(findFile(clientPrivateKeyFile)), new JcaKeyFingerprintCalculator());
        clientPgpSecretKey = PGPExampleUtil
                .readSecretKey(ProjectResource.RESOURCE_PATH.concat(clientPrivateKeyFile));
        clientPgpPrivateKey = PGPExampleUtil.findSecretKey(pgpSec, clientPgpSecretKey.getKeyID(),
                clientPrivateKeyPWD.toCharArray());
        List<PGPPublicKey> pgpPublicKeys = new ArrayList<PGPPublicKey>();
        pgpPublicKeys.add(hsbcPgpPublicKey);
        String clientKeyId = (Long.toHexString(clientPgpPrivateKey.getKeyID()).toUpperCase());

        // 2.2. Encrypts Request Payload with HSBC Public Key
        String requestPayloadEncryptedBase64String = PGPEncryptionExample
                .encrpytRequestPayload(requestPayloadPlainString, pgpPublicKeys, true, true, false);

        // 3. Prepares Digital Signature (JSON Web Token)
        String APIProfileId;
        switch (targetHost) {
            case "SANDBOX":
                APIProfileId = ClientConfig.Sandbox.API_PROFILE_ID;
                break;
            case "UAT":
                APIProfileId = ClientConfig.Uat.API_PROFILE_ID;
            case "PRODUCTION":
                APIProfileId = ClientConfig.Production.API_PROFILE_ID;
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + targetHost);
        }
        System.out.println("\n#3. Start of preparing the Digital Signature (JSON Web Token)");
        // 3.1. Loads Client Private PGP Keys

        // 3.2. Converts Client PGP Private key to RSA key
        PrivateKey clientPrivateKey = new JcaPGPKeyConverter().getPrivateKey(clientPgpPrivateKey);

        // 3.3. Generates token - JWT
        String token = BearerToken.createToken(ClientConfig.TOKEN_TYP, clientKeyId, ClientConfig.TOKEN_ALG, APIProfileId, ClientConfig.TOKEN_AUD, ClientConfig.TOKEN_VER, requestPayloadEncryptedBase64String, ClientConfig.TOKEN_PAYLOAD_HASH_ALG, clientPrivateKey);


        // 5. Receives Response Message from HSBC
        String baseUrl;
        switch (targetHost) {
            case "SANDBOX":
                baseUrl = ProjectResource.Sandbox.ENV_URL;
                break;
            case "UAT":
                baseUrl = ProjectResource.Uat.ENV_URL;
                break;
            case "PRODUCTION":
                baseUrl = ProjectResource.Production.ENV_URL;
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + targetHost);
        }
        String responseMessageString = HttpClientUtil
                .doPost(baseUrl.concat(endpointUrl), requestHeaderJson, requestPayloadEncryptedBase64String, token);
        // 4.1 Triggers the request to HSBC API Service

        System.out.println();
        System.out.println("#5. Receiving Response Message from HSBC API Service...");
        System.out.println("#5.1 Response Message (Base64 Encoded):\n" + responseMessageString);
        System.out.println();
        // 6. Decrypts Response Message from HSBC
        System.out.println("#6. Decrypting Response Message from #5.1...");
        String decrypt = PGPEncryptionExample.decryptResponseMessage(clientPgpPrivateKey, responseMessageString);
        String jsonFormat = jsonFormat(decrypt);
        System.out.println("#6.1 Decrypted Response Message:");
        System.out.println(jsonFormat);
        System.out.println("----- #End of " + apiName + " API Sample Code Steps# -----");
    }

    public static InputStream findFile(final String file) {
        return SampleApiRequest.class.getClassLoader().getResourceAsStream(file);
    }

    public static String jsonFormat(String jsonString) {
        JSONObject object = JSONObject.parseObject(jsonString);
        jsonString = JSON.toJSONString(object, SerializerFeature.PrettyFormat, SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteDateUseDateFormat);
        return jsonString;
    }

}
