package util;
/*
 * ***************************************************************
 * , Copyright. HSBC Holdings plc ccyy ALL RIGHTS RESERVED.
 *
 * This software is only to be used for the purpose for which it
 * has been provided. No part of it is to be reproduced,
 * disassembled, transmitted, stored in a retrieval system or
 * translated in any human or computer language in any way or
 * for any other purposes whatsoever without the prior written
 * consent of HSBC Holdings plc.
 * ***************************************************************
 */

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import config.ClientConfig;
import config.ProjectResource;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import javax.xml.bind.DatatypeConverter;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import static com.runner.tradeFinace.ClientSideTokenNEncryption.encryptPayloadAndMakeToken;


public class SampleClient {

    public static void ExecuteRequest(String apiName, String endpointUrl, JSONObject requestHeaderJson, String unencryptedRequest) throws Exception {

        String targetHost = ClientConfig.targetHost;
        System.out.println("----- #Start of " + apiName + " API Sample Code Steps# -----");
        // 1. Prepare Request to HSBC API to HSBC API Services
        System.out.println("#1. Reading and converting Request JSON File to JSONString...");
        // 1.1. Reads Plain Request JSON
        JSONObject requestJson = JsonUtil.getRequestJson(unencryptedRequest);

        // 1.2. Converts JSON to STRING
        String unEncryptedRequest = JSON.toJSONString(requestJson);
        System.out.println(unEncryptedRequest);
        System.out.println();

        // 2. Encrypts Request Payload
        Map<String, String> apiRequest = encryptPayloadAndMakeToken(requestHeaderJson, unEncryptedRequest);
        String requestEndpointUrl = "";
        switch (targetHost) {
            case "SANDBOXPROD":
                requestEndpointUrl = ProjectResource.Sandbox.ENV_URL.concat(endpointUrl);

        }
        String responseMessageString = HttpClientUtil
                .doPost(requestEndpointUrl, requestHeaderJson, apiRequest.get("requestPayloadEncryptedBase64String"), apiRequest.get("token"));
        System.out.println(requestHeaderJson.getString("schemaVersion"));
        System.out.println("#5. Receiving Response Message from HSBC API Service...");
        System.out.println("Response Message (Base64 Encoded): " + responseMessageString);
        System.out.println();
        // 6. Decrypts Response Message from HSBC
        System.out.println("#6. Decrypting Response Message from HSBC...");
        //String decrypt = PGPEncryptionExample.decryptResponseMessage(clientPgpPrivateKey, responseMessageString);
        //String jsonFormat = jsonFormat(decrypt);
        System.out.println("Decrypted Response Message:");
        //System.out.println(jsonFormat);
        System.out.println("----- #End of " + apiName + " API Sample Code Steps# -----");
    }

    /**
     * createToken
     *
     * @param typ              tokenType
     * @param kid              keyId
     * @param alg              signatureAlgorithm
     * @param sub              subject (HSBC Profile ID)
     * @param aud              audience (HSBC MKT)
     * @param ver              version (1.0)
     * @param encryptedPayload encryptedPayload (By HSBC Public Key and Base 64 Encoded)
     * @param payload_hash_alg algorithm used to hash the encrypted payload
     * @param privateKey       clientPrivateKey (Convert from PGP to RSA)
     * @return token
     * @throws NoSuchAlgorithmException NoSuchAlgorithmException
     */

    public static String createToken(String typ, String kid, SignatureAlgorithm alg, String sub, String aud, String ver, String encryptedPayload, String payload_hash_alg,
                                     PrivateKey privateKey) throws NoSuchAlgorithmException {

        // Generate unique ID for this API call
        String jwtId = UUID.randomUUID().toString();
        // Build JWT
        long nowSeconds = System.currentTimeMillis() / 1000;
        Map<String, Object> claims = new HashMap<String, Object>();
        claims.put("sub", sub);
        claims.put("iat", nowSeconds);
        claims.put("aud", aud);
        claims.put("jti", jwtId);
        String payloadHash = generateMessageDigest(encryptedPayload, payload_hash_alg);
        claims.put("payload_hash", payloadHash);
        claims.put("payload_hash_alg", payload_hash_alg);

        Map<String, Object> header = new HashMap<String, Object>();
        header.put("typ", typ);
        header.put("kid", kid);
        header.put("alg", alg.getValue());
        header.put("ver", ver);

        System.out.println("{");
        System.out.println("\"ver\": " + "\"" + ver + "\",");
        System.out.println("\"kid\": " + "\"" + kid + "\",");
        System.out.println("\"typ\": " + "\"" + typ + "\",");
        System.out.println("\"alg\": " + "\"" + alg.getValue() + "\",");
        System.out.println("\"aud\": " + "\"" + aud + "\",");
        System.out.println("\"sub\": " + "\"" + sub + "\",");
        System.out.println("\"iat\": " + "\"" + nowSeconds + "\",");
        System.out.println("\"jti\": " + "\"" + jwtId + "\",");
        System.out.println("\"payload_hash_alg\": " + "\"" + payload_hash_alg + "\",");
        System.out.println("\"payload_hash\": " + "\"" + payloadHash + "\"");
        System.out.println("}");
        System.out.println();

        String token = Jwts.builder().setHeader(header)
                .setClaims(claims).signWith(privateKey, alg).compact();
        return token;
    }

    // Helper method for getting message digest
    private static String generateMessageDigest(String message, String hashingAlgorithm)
            throws NoSuchAlgorithmException {
        return DatatypeConverter.printHexBinary(MessageDigest.getInstance(hashingAlgorithm).digest(message.getBytes()))
                .toLowerCase();
    }

    public static InputStream findFile(final String file) {
        return SampleClient.class.getClassLoader().getResourceAsStream(file);
    }

    public static String jsonFormat(String jsonString) {
        JSONObject object = JSONObject.parseObject(jsonString);
        jsonString = JSON.toJSONString(object, SerializerFeature.PrettyFormat, SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteDateUseDateFormat);
        return jsonString;
    }

}
